@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2011 by The BRLTTY Developers.
+ * Copyright (C) 1995-2013 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -258,13 +258,14 @@ qflush (void)
 
 static int
 await_ack (void) {
-  hasTimedOut(0);
+  TimePeriod period;
+  startTimePeriod(&period, ACK_TIMEOUT);
   waiting_ack = 1;
   do {
     approximateDelay(10);	/* sleep for 10 ms */
     qfill();
     if (!waiting_ack) return 1;
-  } while (!hasTimedOut(ACK_TIMEOUT));
+  } while (!afterTimePeriod(&period, NULL));
   return 0;
 }
 
@@ -465,7 +466,7 @@ brl_writeWindow (BrailleDisplay * brl, const wchar_t *text)
   }
 
   /* Only refresh display if the data has changed: */
-  if (cellsHaveChanged(prevdata, rawdata, blitesz, NULL, NULL))
+  if (cellsHaveChanged(prevdata, rawdata, blitesz, NULL, NULL, NULL))
     {
       /* Dot mapping from standard to BrailleLite: */
       translateOutputCells(rawdata, rawdata, blitesz);
@@ -540,7 +541,7 @@ brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context)
 	  /* I thought I was smart when I suggested to remove CMD_CUT_END.
 	     Well now there's this nasty exception: the command offset
 	     depends on the display size! */
-	  if(key.cmd == BRL_BLK_CUTRECT || key.cmd == BRL_BLK_CUTLINE)
+	  if(key.cmd == BRL_BLK_COPY_RECT || key.cmd == BRL_BLK_COPY_LINE)
 	    key.cmd += blitesz-1;
 
 	  if(key.spcbar && (key.cmd &BRL_MSK_BLK) == BRL_BLK_PASSKEY) {
@@ -584,8 +585,8 @@ brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context)
 	    dot8shift = 0;
 #endif /* USE_TEXTTRANS */
 	    if(kbemu)
-	      message (NULL, "keyboard emu on", MSG_SILENT);
-	    else message (NULL, "keyboard emu off", MSG_SILENT);
+	      message (NULL, gettext("keyboard emu on"), MSG_SILENT);
+	    else message (NULL, gettext("keyboard emu off"), MSG_SILENT);
 	    return BRL_CMD_NOOP;
 	  case BLT_ROTATE:	/* rotate Braille Lite by 180 degrees */
 	    reverse_kbd ^= 1;
@@ -596,13 +597,13 @@ brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context)
 	    return BRL_CMD_NOOP;
 	  case BLT_REPEAT:	/* set repeat count */
 	    hold = 0;
-	    sprintf (outmsg, "Repeat count:");
+	    snprintf (outmsg, sizeof(outmsg), "%s:", gettext("repeat count"));
 	    message (NULL, outmsg, MSG_SILENT | MSG_NODELAY);
 	    intoverride = 1;
 	    state = ST_REPEAT;
 	    return BRL_CMD_NOOP;
 	  case BLT_CONFIG:	/* configuration menu */
-	    sprintf (outmsg, "Config? [m/s/r/z]");
+	    snprintf (outmsg, sizeof(outmsg), "%s? [m/s/r/z]", gettext("config"));
 	    message (NULL, outmsg, MSG_SILENT | MSG_NODELAY);
 	    intoverride = 1;
 	    state = ST_CONFIG;
@@ -663,7 +664,7 @@ brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context)
 	    return BRL_CMD_NOOP;
 	  case BLT_ABORT:	/* abort - quit keyboard emulation */
 	    kbemu = 0;
-	    message (NULL, "keyboard emu off", MSG_SILENT);
+	    message (NULL, gettext("keyboard emu off"), MSG_SILENT);
 	    return BRL_CMD_NOOP;
 	  default:		/* unrecognised command */
 	    shift = shiftlck = ctrl = meta = 0;
@@ -736,8 +737,8 @@ brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context)
 	      state = ST_NORMAL;
 	    }
 	  return temp;
-	case BRL_BLK_CUTBEGIN:	/* begin cut */
-	case BRL_BLK_CUTAPPEND:
+	case BRL_BLK_CLIP_NEW:	/* begin copy */
+	case BRL_BLK_CLIP_ADD:
 	  if (key.spcbar)
 	    {
 	      temp = key.cmd + int_cursor - 1;
@@ -745,8 +746,8 @@ brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context)
 	      state = ST_NORMAL;
 	    }
 	  return temp;
-	case BRL_BLK_CUTRECT:	/* end cut */
-	case BRL_BLK_CUTLINE:
+	case BRL_BLK_COPY_RECT:	/* end copy */
+	case BRL_BLK_COPY_LINE:
 	  if (key.spcbar)
 	    {
 	      temp = key.cmd + int_cursor - 1;
@@ -774,9 +775,11 @@ brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context)
       if (key.asc >= '0' && key.asc <= '9')
 	{
 	  hold = (hold * 10 + key.asc - '0') % 100;
-	  if(hold)
-	    sprintf (outmsg, "Repeat count: %d", hold);
-	  else sprintf (outmsg, "Repeat count: ");
+	  if (hold) {
+	    snprintf (outmsg, sizeof(outmsg), "%s: %d", gettext("repeat count"), hold);
+	  } else {
+            snprintf (outmsg, sizeof(outmsg), "%s: ", gettext("repeat count"));
+          }
 	  intoverride = 0;
 	  message (NULL, outmsg, MSG_SILENT | MSG_NODELAY);
 	  intoverride = 1;
@@ -784,7 +787,7 @@ brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context)
       else if (key.routing)
 	{
 	  hold = key.routing +1;
-	  sprintf (outmsg, "Repeat count: %d", hold);
+	  snprintf (outmsg, sizeof(outmsg), "%s: %d", gettext("repeat count"), hold);
 	  intoverride = 0;
 	  message (NULL, outmsg, MSG_SILENT | MSG_NODELAY);
 	  intoverride = 1;
