@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2011 by The BRLTTY Developers.
+ * Copyright (C) 1995-2013 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -18,7 +18,6 @@
 
 #include "prologue.h"
 
-#include <math.h>
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
@@ -28,7 +27,6 @@
 #include "log.h"
 #include "parse.h"
 #include "timing.h"
-#include "misc.h"
 #include "ascii.h"
 
 typedef enum {
@@ -393,10 +391,6 @@ static unsigned char rawStatus[MAX_STCELLS];		/* to hold status info */
 static unsigned char prevStatus[MAX_STCELLS];	/* to hold previous status */
 static const ModelEntry *model;		/* points to terminal model config struct */
 
-static unsigned char *at2Buffer;
-static int at2Size;
-static int at2Count;
-
 typedef struct {
   int (*openPort) (char **parameters, const char *device);
   void (*closePort) ();
@@ -481,12 +475,12 @@ typedef enum {
   HT_HID_CMD_FlushBuffers = 0X01, /* flush input and output buffers */
 } HtHidCommand;
 
-static uint32_t hidReportSize_OutData;
-static uint32_t hidReportSize_InData;
-static uint32_t hidReportSize_InCommand;
-static uint32_t hidReportSize_OutVersion;
-static uint32_t hidReportSize_OutBaud;
-static uint32_t hidReportSize_InBaud;
+static size_t hidReportSize_OutData;
+static size_t hidReportSize_InData;
+static size_t hidReportSize_InCommand;
+static size_t hidReportSize_OutVersion;
+static size_t hidReportSize_OutBaud;
+static size_t hidReportSize_InBaud;
 
 static uint16_t hidFirmwareVersion;
 static unsigned char *hidInputReport = NULL;
@@ -516,7 +510,7 @@ setHidReport (const unsigned char *report, int size) {
 
 typedef struct {
   HT_HidReportNumber number;
-  uint32_t *size;
+  size_t *size;
 } ReportEntry;
 
 static void
@@ -635,10 +629,10 @@ initializeUsb2 (void) {
 static int
 awaitUsbInput2 (int milliseconds) {
   if (hidReportSize_OutData) {
-    struct timeval startTime;
+    TimePeriod period;
 
     if (hidInputOffset < hidInputLength) return 1;
-    gettimeofday(&startTime, NULL);
+    startTimePeriod(&period, milliseconds);
 
     while (1) {
       int result = getHidReport(HT_HID_RPT_OutData, hidInputReport,
@@ -648,7 +642,7 @@ awaitUsbInput2 (int milliseconds) {
       hidInputOffset = 0;
       if (hidInputLength > 0) return 1;
 
-      if (millisecondsSince(&startTime) >= milliseconds) break;
+      if (afterTimePeriod(&period, NULL)) break;
       approximateDelay(10);
     }
   }
@@ -726,10 +720,10 @@ initializeUsb3 (void) {
 static int
 awaitUsbInput3 (int milliseconds) {
   if (hidReportSize_OutData) {
-    struct timeval startTime;
+    TimePeriod period;
 
     if (hidInputOffset < hidInputLength) return 1;
-    gettimeofday(&startTime, NULL);
+    startTimePeriod(&period, milliseconds);
 
     while (1) {
       int result = usbReapInput(usb->device, usb->definition.inputEndpoint,
@@ -737,10 +731,12 @@ awaitUsbInput3 (int milliseconds) {
                                 0, 100);
 
       if (result == -1) return 0;
-      hidInputOffset = 0;
-      if (hidInputLength > 0) return 1;
+      if (result > 0 && hidInputLength > 0) {
+        hidInputOffset = 0;
+        return 1;
+      }
 
-      if (millisecondsSince(&startTime) >= milliseconds) break;
+      if (afterTimePeriod(&period, NULL)) break;
       approximateDelay(10);
     }
   }
@@ -786,10 +782,8 @@ static const UsbOperations usbOperations3 = {
 static int
 openUsbPort (char **parameters, const char *device) {
   const SerialParameters serial = {
+    SERIAL_DEFAULT_PARAMETERS,
     .baud = baud,
-    .flowControl = SERIAL_FLOW_NONE,
-    .dataBits = 8,
-    .stopBits = 1,
     .parity = SERIAL_PARITY_ODD
   };
 
@@ -811,25 +805,25 @@ openUsbPort (char **parameters, const char *device) {
     }
     ,
     { /* Easy Braille (HID) */
-      .vendor=HT_USB_VENDOR, .product=HT_MODEL_EasyBraille,
+      .vendor=0X1FE4, .product=0X0044,
       .configuration=1, .interface=0, .alternative=0,
       .data=&usbOperations2
     }
     ,
     { /* Braille Star 40 (HID) */
-      .vendor=HT_USB_VENDOR, .product=HT_MODEL_BrailleStar40,
+      .vendor=0X1FE4, .product=0X0074,
       .configuration=1, .interface=0, .alternative=0,
       .data=&usbOperations2
     }
     ,
     { /* USB-HID adapter */
-      .vendor=HT_USB_VENDOR, .product=HT_MODEL_UsbHidAdapter,
+      .vendor=0X1FE4, .product=0X0003,
       .configuration=1, .interface=0, .alternative=0,
       .data=&usbOperations2
     }
     ,
     { /* Active Braille */
-      .vendor=HT_USB_VENDOR, .product=HT_MODEL_ActiveBraille,
+      .vendor=0X1FE4, .product=0X0054,
       .configuration=1, .interface=0, .alternative=0,
       .inputEndpoint=1, .outputEndpoint=1,
       .data=&usbOperations3
@@ -941,7 +935,7 @@ typedef enum {
   BDS_WRITING
 } BrailleDisplayState;
 static BrailleDisplayState currentState = BDS_OFF;
-static struct timeval stateTime;
+static TimePeriod statePeriod;
 static unsigned int retryCount = 0;
 static unsigned char updateRequired = 0;
 
@@ -1043,7 +1037,8 @@ setState (BrailleDisplayState state) {
     retryCount = 0;
     currentState = state;
   }
-  gettimeofday(&stateTime, NULL);
+
+  startTimePeriod(&statePeriod, 1000);
   // logMessage(LOG_DEBUG, "State: %d+%d", currentState, retryCount);
 }
 
@@ -1183,7 +1178,7 @@ logDateTime (BrailleDisplay *brl, const HT_DateTime *dateTime) {
              " %04" PRIu16 "-%02" PRIu8 "-%02" PRIu8
              " %02" PRIu8 ":%02" PRIu8 ":%02" PRIu8,
              model->name,
-             getBigEndian(dateTime->year), dateTime->month, dateTime->day,
+             getBigEndian16(dateTime->year), dateTime->month, dateTime->day,
              dateTime->hour, dateTime->minute, dateTime->second);
 
   return 1;
@@ -1191,33 +1186,44 @@ logDateTime (BrailleDisplay *brl, const HT_DateTime *dateTime) {
 
 static int
 synchronizeDateTime (BrailleDisplay *brl, const HT_DateTime *dateTime) {
-  struct tm t0 = {
-    .tm_year = getBigEndian(dateTime->year) - 1900,
-    .tm_mon = dateTime->month - 1,
-    .tm_mday = dateTime->day,
-    .tm_hour = dateTime->hour,
-    .tm_min = dateTime->minute,
-    .tm_sec = dateTime->second,
-    .tm_isdst = -1
-  };
+  long int delta;
+  TimeValue hostTime;
+  getCurrentTime(&hostTime);
 
-  time_t deviceTime = mktime(&t0);
-  time_t hostTime = time(NULL);
-  double delta = difftime(deviceTime, hostTime);
+  {
+    TimeValue deviceTime;
 
-  if (fabs(delta) > 1) {
-    const struct tm *t = localtime(&hostTime);
+    {
+      TimeComponents components = {
+        .year = getBigEndian16(dateTime->year),
+        .month = dateTime->month - 1,
+        .day = dateTime->day - 1,
+        .hour = dateTime->hour,
+        .minute = dateTime->minute,
+        .second = dateTime->second
+      };
 
-    HT_DateTime payload = {
-      .month = t->tm_mon + 1,
-      .day = t->tm_mday,
-      .hour = t->tm_hour,
-      .minute = t->tm_min,
-      .second = t->tm_sec
-    };
-    putLittleEndian(&payload.year, t->tm_year + 1900);
+      makeTimeValue(&deviceTime, &components);
+    }
 
-    logMessage(LOG_DEBUG, "Time difference between host and device: %.0f", delta);
+    delta = millisecondsBetween(&hostTime, &deviceTime);
+    if (delta < 0) delta = -delta;
+  }
+
+  if (delta > 1000) {
+    TimeComponents components;
+    HT_DateTime payload;
+
+    expandTimeValue(&hostTime, &components);
+    putLittleEndian16(&payload.year, components.year);
+    payload.month = components.month + 1;
+    payload.day = components.day + 1;
+    payload.hour = components.hour;
+    payload.minute = components.minute;
+    payload.second = components.second;
+
+    logMessage(LOG_DEBUG, "Time difference between host and device: %ld.%03ld",
+               (delta / MSECS_PER_SEC), (delta % MSECS_PER_SEC));
 
     if (writeExtendedPacket(brl, HT_EXTPKT_SetRTC,
                             (unsigned char *)&payload, sizeof(payload))) {
@@ -1232,10 +1238,6 @@ static int
 brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
   unsigned int setTime = 0;
 
-  at2Buffer = NULL;
-  at2Size = 0;
-  at2Count = 0;
-
   if (isSerialDevice(&device)) {
     io = &serialOperations;
   } else if (isUsbDevice(&device)) {
@@ -1308,11 +1310,6 @@ brl_destruct (BrailleDisplay *brl) {
   }
   io->closePort();
 
-  if (at2Buffer) {
-    free(at2Buffer);
-    at2Buffer = NULL;
-  }
-
   deallocateBuffers();
 }
 
@@ -1367,7 +1364,7 @@ updateCells (BrailleDisplay *brl) {
 static int
 brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
   const size_t cellCount = model->textCells;
-  if (cellsHaveChanged(prevData, brl->buffer, cellCount, NULL, NULL)) {
+  if (cellsHaveChanged(prevData, brl->buffer, cellCount, NULL, NULL, NULL)) {
     translateOutputCells(rawData, prevData, cellCount);
     updateRequired = 1;
   }
@@ -1378,7 +1375,7 @@ brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
 static int
 brl_writeStatus (BrailleDisplay *brl, const unsigned char *st) {
   const size_t cellCount = model->statusCells;
-  if (cellsHaveChanged(prevStatus, st, cellCount, NULL, NULL)) {
+  if (cellsHaveChanged(prevStatus, st, cellCount, NULL, NULL, NULL)) {
     translateOutputCells(rawStatus, prevStatus, cellCount);
     updateRequired = 1;
   }
@@ -1445,12 +1442,6 @@ static int
 brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context) {
   int noInput = 1;
 
-  if (at2Count) {
-    unsigned char code = at2Buffer[0];
-    memmove(at2Buffer, at2Buffer+1, --at2Count);
-    return BRL_BLK_PASSAT + code;
-  }
-
   while (1) {
     HT_Packet packet;
     int size = brl_readPacket(brl, &packet, sizeof(packet));
@@ -1546,20 +1537,8 @@ brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context) {
                     break;
 
                   case HT_EXTPKT_Scancode: {
-                    if (length) {
-                      unsigned char code = bytes++[0];
-                      if (--length) {
-                        int newCount = at2Count + length;
-                        if (newCount > at2Size) {
-                          int newSize = (newCount | 0XF) + 1;
-                          at2Buffer = reallocWrapper(at2Buffer, newSize);
-                          at2Size = newSize;
-                        }
-                        memcpy(at2Buffer+at2Count, bytes, length);
-                        at2Count = newCount;
-                      }
-                      return BRL_BLK_PASSAT + code;
-                    }
+                    while (length--)
+                      enqueueCommand(BRL_BLK_PASSAT | BRL_ARG(*bytes++));
                     break;
                   }
 
@@ -1666,7 +1645,7 @@ brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context) {
         break;
 
       case BDS_WRITING:
-        if (millisecondsSince(&stateTime) > 1000) {
+        if (afterTimePeriod(&statePeriod, NULL)) {
           if (retryCount > 3) return BRL_CMD_RESTARTBRL;
           if (!writeCells(brl)) return BRL_CMD_RESTARTBRL;
         }
