@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2012 by The BRLTTY Developers.
+ * Copyright (C) 1995-2013 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -492,7 +492,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
   charactersPerSecond = serialBaud / serialGetCharacterBits(serialDevice);
 
   /* Mark time of last command to initialize typematic watch */
-  getCurrentTime(&last_ping);
+  getMonotonicTime(&last_ping);
   last_readbrl_time = last_ping;
   lastcmd_time = last_ping;
   lastcmd = EOF;
@@ -867,14 +867,13 @@ brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context)
   unsigned int code = 0; /* 32bits code representing pressed keys once the
 			    input bytes are interpreted */
   int res = EOF; /* command code to return. code is mapped to res. */
-  static int pending_cmd = EOF;
   char buf[MAXREAD], /* read buffer */
        packtype = 0; /* type of packet being received (state) */
   unsigned i;
   TimeValue now;
   int skip_this_cmd = 0;
 
-  getCurrentTime(&now);
+  getMonotonicTime(&now);
   if (millisecondsBetween(&last_readbrl_time, &now) > READBRL_SKIP_TIME)
     /* if the key we get this time is the same as the one we returned at last
        call, and if it has been abnormally long since we were called
@@ -884,13 +883,6 @@ brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context)
     skip_this_cmd = 1;
   last_readbrl_time = now;
        
-  if(pending_cmd != EOF){
-    res = pending_cmd;
-    lastcmd = EOF;
-    pending_cmd = EOF;
-    return(res);
-  }
-
 /* Key press codes come in pairs of bytes for nav and pb40, in 6bytes
    for pb65/80. Each byte has bits representing individual keys + a special
    mask/signature in the most significant 3bits.
@@ -924,13 +916,13 @@ brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context)
 	  approximateDelay(SEND_DELAY);
 	}
 	pings++;
-	getCurrentTime(&last_ping_sent);
+	getMonotonicTime(&last_ping_sent);
       }
     }
     return (EOF);
   }
   /* there was some input, we heard something. */
-  getCurrentTime(&last_ping);
+  getMonotonicTime(&last_ping);
   pings=0;
 
 #ifdef RECV_DELAY
@@ -1128,8 +1120,7 @@ brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context)
 	}
 #endif /* 0 */
       else if(sw_howmany == 3 && sw_which[0]+2 == sw_which[1]){
-	  res = BRL_BLK_CLIP_NEW + sw_which[0];
-	  pending_cmd = BRL_BLK_COPY_RECT + sw_which[2];
+	  res = BRL_BLK_CLIP_COPY | BRL_ARG(sw_which[0]) | BRL_EXT(sw_which[2]);
 	}
     }
   else switch (code){
@@ -1258,11 +1249,11 @@ brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context)
   /* If this is a typematic repetition of some key other than movement keys */
   if (lastcmd == res && !is_repeat_cmd (res)){
     if(skip_this_cmd){
-      getCurrentTime(&lastcmd_time);
+      getMonotonicTime(&lastcmd_time);
       res = EOF;
     }else{
       /* if to short a time has elapsed since last command, ignore this one */
-      getCurrentTime(&now);
+      getMonotonicTime(&now);
       if (millisecondsBetween(&lastcmd_time, &now) < NONREPEAT_TIMEOUT)
 	res = EOF;
     }
@@ -1270,7 +1261,7 @@ brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context)
   /* reset timer to avoid unwanted typematic */
   if (res != EOF){
     lastcmd = res;
-    getCurrentTime(&lastcmd_time);
+    getMonotonicTime(&lastcmd_time);
   }
 
   /* Special: */
