@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2011 by The BRLTTY Developers.
+ * Copyright (C) 1995-2013 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -30,7 +30,7 @@ typedef enum {
   CTB_CAP_NONE,
   CTB_CAP_SIGN,
   CTB_CAP_DOT7
-} CTB_CapitalizationMode;;
+} CTB_CapitalizationMode;
 
 #ifdef __cplusplus
 }
